import * as tslib_1 from "tslib";
import * as imm from "object-path-immutable";
import { createAction, handleActions } from "redux-actions";
import actionTypes from "./constants";
import { Observable } from "./rxjs-imports";
import { addLinksToState, ensureResourceTypeInState, removeResourceFromState, setIsInvalidatingForExistingResource, updateOrInsertResourcesIntoState } from "./state-mutation";
import { apiRequest } from "./utils";
var API_SET_ENDPOINT_HOST = actionTypes.API_SET_ENDPOINT_HOST, API_SET_ENDPOINT_PATH = actionTypes.API_SET_ENDPOINT_PATH, API_SET_HEADERS = actionTypes.API_SET_HEADERS, API_SET_HEADER = actionTypes.API_SET_HEADER, API_HYDRATE = actionTypes.API_HYDRATE, API_WILL_CREATE = actionTypes.API_WILL_CREATE, API_CREATED = actionTypes.API_CREATED, API_CREATE_FAILED = actionTypes.API_CREATE_FAILED, API_WILL_READ = actionTypes.API_WILL_READ, API_READ = actionTypes.API_READ, API_READ_FAILED = actionTypes.API_READ_FAILED, API_WILL_UPDATE = actionTypes.API_WILL_UPDATE, API_UPDATED = actionTypes.API_UPDATED, API_UPDATE_FAILED = actionTypes.API_UPDATE_FAILED, API_WILL_DELETE = actionTypes.API_WILL_DELETE, API_DELETED = actionTypes.API_DELETED, API_DELETE_FAILED = actionTypes.API_DELETE_FAILED;
export var IS_DELETING = "IS_DELETING";
export var IS_UPDATING = "IS_UPDATING";
export var setEndpointHost = createAction(API_SET_ENDPOINT_HOST);
export var setEndpointPath = createAction(API_SET_ENDPOINT_PATH);
export var setHeaders = createAction(API_SET_HEADERS);
export var setHeader = createAction(API_SET_HEADER);
export var hydrateStore = createAction(API_HYDRATE);
var apiWillCreate = createAction(API_WILL_CREATE);
var apiCreated = createAction(API_CREATED);
var apiCreateFailed = createAction(API_CREATE_FAILED);
var apiWillRead = createAction(API_WILL_READ);
var apiRead = createAction(API_READ);
var apiReadFailed = createAction(API_READ_FAILED);
var apiWillUpdate = createAction(API_WILL_UPDATE);
var apiUpdated = createAction(API_UPDATED);
var apiUpdateFailed = createAction(API_UPDATE_FAILED);
var apiWillDelete = createAction(API_WILL_DELETE);
var apiDeleted = createAction(API_DELETED);
var apiDeleteFailed = createAction(API_DELETE_FAILED);
export var setAccessToken = function (at) {
    return function (dispatch) {
        dispatch(setHeader({ Authorization: "Bearer " + at }));
    };
};
export var createResource = function (resource) {
    return function (dispatch, getState) {
        dispatch(apiWillCreate(resource));
        var _a = getState().api.endpoint, apiHost = _a.host, apiPath = _a.path, headers = _a.headers;
        var endpoint = "" + apiHost + apiPath + "/" + resource.type;
        return apiRequest(endpoint, {
            headers: headers,
            method: "POST",
            withCredentials: true,
            body: JSON.stringify({
                data: resource
            })
        })
            .map(function (json) {
            dispatch(apiCreated(json));
            return json;
        })
            .catch(function (error) {
            var err = error;
            err.resource = resource;
            dispatch(apiCreateFailed(err));
            return Observable.throw(err);
        });
    };
};
export var readEndpoint = function (endpoint, _a) {
    var _b = (_a === void 0 ? {} : _a).options, options = _b === void 0 ? {
        indexLinks: undefined
    } : _b;
    return function (dispatch, getState) {
        dispatch(apiWillRead(endpoint));
        var _a = getState().api.endpoint, apiHost = _a.host, apiPath = _a.path, headers = _a.headers;
        var apiEndpoint = "" + apiHost + apiPath + "/" + endpoint;
        return apiRequest("" + apiEndpoint, {
            headers: headers,
            withCredentials: true
        })
            .map(function (json) {
            dispatch(apiRead(tslib_1.__assign({ endpoint: endpoint, options: options }, json)));
            return json;
        })
            .catch(function (error) {
            var err = error;
            err.endpoint = endpoint;
            dispatch(apiReadFailed(err));
            return Observable.throw(err);
        });
    };
};
export var updateResource = function (resource) {
    return function (dispatch, getState) {
        dispatch(apiWillUpdate(resource));
        var _a = getState().api.endpoint, apiHost = _a.host, apiPath = _a.path, headers = _a.headers;
        var endpoint = "" + apiHost + apiPath + "/" + resource.type + "/" + resource.id;
        return apiRequest(endpoint, {
            headers: headers,
            method: "PATCH",
            withCredentials: true,
            body: JSON.stringify({
                data: resource
            })
        })
            .map(function (json) {
            dispatch(apiUpdated(json));
            return json;
        })
            .catch(function (error) {
            var err = error;
            err.resource = resource;
            dispatch(apiUpdateFailed(err));
            return Observable.throw(err);
        });
    };
};
export var deleteResource = function (resource) {
    return function (dispatch, getState) {
        dispatch(apiWillDelete(resource));
        var _a = getState().api.endpoint, apiHost = _a.host, apiPath = _a.path, headers = _a.headers;
        var endpoint = "" + apiHost + apiPath + "/" + resource.type + "/" + resource.id;
        return apiRequest(endpoint, {
            headers: headers,
            method: "DELETE",
            withCredentials: true
        })
            .do(function () {
            dispatch(apiDeleted(resource));
        })
            .catch(function (error) {
            var err = error;
            err.resource = resource;
            dispatch(apiDeleteFailed(err));
            return Observable.throw(err);
        });
    };
};
export var requireResource = function (resourceType, endpoint) {
    if (endpoint === void 0) { endpoint = resourceType; }
    return function (dispatch, getState) {
        var api = getState().api;
        if (api.hasOwnProperty(resourceType)) {
            return Observable.of();
        }
        return dispatch(readEndpoint(endpoint));
    };
};
export var reducer = handleActions((_a = {},
    _a[API_SET_HEADERS] = function (state, _a) {
        var headers = _a.payload;
        return imm(state)
            .set(["endpoint", "headers"], headers)
            .value();
    },
    _a[API_SET_HEADER] = function (state, _a) {
        var header = _a.payload;
        var newState = imm(state);
        Object.keys(header).forEach(function (key) {
            newState.set(["endpoint", "headers", key], header[key]);
        });
        return newState.value();
    },
    _a[API_SET_ENDPOINT_HOST] = function (state, _a) {
        var host = _a.payload;
        return imm(state)
            .set(["endpoint", "host"], host)
            .value();
    },
    _a[API_SET_ENDPOINT_PATH] = function (state, _a) {
        var path = _a.payload;
        return imm(state)
            .set(["endpoint", "path"], path)
            .value();
    },
    _a[API_HYDRATE] = function (state, _a) {
        var resources = _a.payload;
        var entities = Array.isArray(resources.data)
            ? resources.data
            : [resources.data];
        var newState = updateOrInsertResourcesIntoState(state, entities.concat(resources.included || []));
        return imm(newState).value();
    },
    _a[API_WILL_CREATE] = function (state) {
        return imm(state)
            .set(["isCreating"], state.isCreating + 1)
            .value();
    },
    _a[API_CREATED] = function (state, _a) {
        var resources = _a.payload;
        var entities = Array.isArray(resources.data)
            ? resources.data
            : [resources.data];
        var newState = updateOrInsertResourcesIntoState(state, entities.concat(resources.included || []));
        return imm(newState)
            .set("isCreating", state.isCreating - 1)
            .value();
    },
    _a[API_CREATE_FAILED] = function (state) {
        return imm(state)
            .set(["isCreating"], state.isCreating - 1)
            .value();
    },
    _a[API_WILL_READ] = function (state) {
        return imm(state)
            .set(["isReading"], state.isReading + 1)
            .value();
    },
    _a[API_READ] = function (state, _a) {
        var payload = _a.payload;
        var resources = (Array.isArray(payload.data)
            ? payload.data
            : [payload.data]).concat(payload.included || []);
        var newState = updateOrInsertResourcesIntoState(state, resources);
        var finalState = addLinksToState(newState, payload.links, payload.options);
        return imm(finalState)
            .set("isReading", state.isReading - 1)
            .value();
    },
    _a[API_READ_FAILED] = function (state) {
        return imm(state)
            .set(["isReading"], state.isReading - 1)
            .value();
    },
    _a[API_WILL_UPDATE] = function (state, _a) {
        var resource = _a.payload;
        var type = resource.type, id = resource.id;
        var newState = ensureResourceTypeInState(state, type);
        return setIsInvalidatingForExistingResource(newState, { type: type, id: id }, IS_UPDATING)
            .set("isUpdating", state.isUpdating + 1)
            .value();
    },
    _a[API_UPDATED] = function (state, _a) {
        var resources = _a.payload;
        var entities = Array.isArray(resources.data)
            ? resources.data
            : [resources.data];
        var newState = updateOrInsertResourcesIntoState(state, entities.concat(resources.included || []));
        return imm(newState)
            .set("isUpdating", state.isUpdating - 1)
            .value();
    },
    _a[API_UPDATE_FAILED] = function (state, _a) {
        var resource = _a.payload.resource;
        var type = resource.type, id = resource.id;
        return setIsInvalidatingForExistingResource(state, { type: type, id: id }, IS_UPDATING)
            .set("isUpdating", state.isUpdating + 1)
            .value();
    },
    _a[API_WILL_DELETE] = function (state, _a) {
        var resource = _a.payload;
        var type = resource.type, id = resource.id;
        return setIsInvalidatingForExistingResource(state, { type: type, id: id }, IS_DELETING)
            .set("isDeleting", state.isDeleting + 1)
            .value();
    },
    _a[API_DELETED] = function (state, _a) {
        var resource = _a.payload;
        return removeResourceFromState(state, resource)
            .set("isDeleting", state.isDeleting - 1)
            .value();
    },
    _a[API_DELETE_FAILED] = function (state, _a) {
        var resource = _a.payload.resource;
        var type = resource.type, id = resource.id;
        return setIsInvalidatingForExistingResource(state, { type: type, id: id }, IS_DELETING)
            .set("isDeleting", state.isDeleting + 1)
            .value();
    },
    _a), {
    isCreating: 0,
    isReading: 0,
    isUpdating: 0,
    isDeleting: 0,
    endpoint: {
        host: null,
        path: null,
        headers: {
            "Content-Type": "application/vnd.api+json",
            Accept: "application/vnd.api+json"
        }
    }
});
var _a;
//# sourceMappingURL=jsonapi.js.map