import * as imm from "object-path-immutable";
import * as pluralize from "pluralize";
import { hasOwnProperties } from "./utils";
var equal = require("deep-equal");
import assign from "lodash-es/assign";
import find from "lodash-es/find";
import findIndex from "lodash-es/findIndex";
export var makeUpdateReverseRelationship = function (resource, relationship, newRelation) {
    if (newRelation === void 0) { newRelation = {
        type: resource.type,
        id: resource.id
    }; }
    return function (foreignResources) {
        var idx = findIndex(foreignResources, function (item) { return item.id === relationship.data.id; });
        if (idx === -1) {
            return foreignResources;
        }
        var _a = [1, 2].map(function (i) { return pluralize(resource.type, i); }), singular = _a[0], plural = _a[1];
        var relCase = find([singular, plural], function (r) {
            return hasOwnProperties(foreignResources[idx], ["relationships", r]);
        });
        if (!relCase) {
            return foreignResources;
        }
        var relPath = ["relationships", relCase, "data"];
        var idxRelPath = [idx].concat(relPath);
        var immutableForeingResources = imm(foreignResources);
        if (!hasOwnProperties(foreignResources[idx], relPath)) {
            return immutableForeingResources.push(idxRelPath, newRelation).value();
        }
        var foreignResourceRel = foreignResources[idx].relationships[relCase].data;
        if ((newRelation &&
            Array.isArray(foreignResourceRel) &&
            findIndex(~foreignResourceRel, function (rel) { return rel.id === newRelation.id && rel.type === newRelation.type; })) ||
            (newRelation &&
                foreignResourceRel &&
                foreignResourceRel.id === newRelation.id &&
                foreignResourceRel.type === newRelation.type)) {
            return foreignResources;
        }
        else if (Array.isArray(foreignResourceRel) && !newRelation) {
            var relIdx = findIndex(foreignResourceRel, function (item) { return item.id === resource.id; });
            if (foreignResourceRel[relIdx]) {
                var deletePath = [idx, "relationships", singular, "data", relIdx];
                return imm(foreignResources)
                    .del(deletePath)
                    .value();
            }
            return foreignResources;
        }
        if (relCase === singular) {
            return immutableForeingResources.set(idxRelPath, newRelation).value();
        }
        return immutableForeingResources.push(idxRelPath, newRelation).value();
    };
};
var stateContainsResource = function (state, resource) {
    var updatePath = [resource.type, "data"];
    if (hasOwnProperties(state, updatePath)) {
        return (findIndex(state[resource.type].data, function (item) { return item.id === resource.id; }) > -1);
    }
    return false;
};
export var addLinksToState = function (state, links, options) {
    if (options === undefined || options.indexLinks === undefined) {
        return state;
    }
    var indexLinkName = options.indexLinks;
    var newState = imm.set(state, "links." + indexLinkName, links);
    return newState;
};
export var updateOrInsertResource = function (state, resource) {
    if (typeof resource !== "object") {
        return state;
    }
    var newState = state;
    var updatePath = [resource.type, "data"];
    if (stateContainsResource(state, resource)) {
        var resources = state[resource.type].data;
        var idx = findIndex(resources, function (item) { return item.id === resource.id; });
        if (resource.relationships) {
            var relationships = {};
            for (var relationship in resource.relationships) {
                if (!resource.relationships[relationship].data) {
                    relationships[relationship] =
                        resources[idx].relationships[relationship];
                }
            }
            assign(resource.relationships, relationships);
        }
        if (!equal(resources[idx], resource)) {
            newState = imm.set(newState, updatePath.concat(idx), resource);
        }
    }
    else {
        newState = imm.push(newState, updatePath, resource);
    }
    var rels = resource.relationships;
    if (!rels) {
        return newState;
    }
    Object.keys(rels).forEach(function (relKey) {
        if (!hasOwnProperties(rels[relKey], ["data", "type"])) {
            return;
        }
        var entityPath = [rels[relKey].data.type, "data"];
        if (!hasOwnProperties(newState, entityPath)) {
            return;
        }
        var updateReverseRelationship = makeUpdateReverseRelationship(resource, rels[relKey]);
        newState = imm.set(newState, entityPath, updateReverseRelationship(newState[rels[relKey].data.type].data));
    });
    return newState;
};
export var removeResourceFromState = function (state, resource) {
    var index = findIndex(state[resource.type].data, function (e) { return e.id === resource.id; });
    var path = [resource.type, "data", index];
    var entityRelationships = resource.relationships || {};
    return Object.keys(entityRelationships).reduce(function (newState, key) {
        if (!resource.relationships[key].data) {
            return newState;
        }
        var entityPath = [resource.relationships[key].data.type, "data"];
        if (hasOwnProperties(state, entityPath)) {
            var updateReverseRelationship = makeUpdateReverseRelationship(resource, resource.relationships[key], null);
            return newState.set(entityPath, updateReverseRelationship(state[resource.relationships[key].data.type].data));
        }
        return newState;
    }, imm(state).del(path));
};
export var updateOrInsertResourcesIntoState = function (state, resources) {
    return resources.reduce(updateOrInsertResource, state);
};
export var setIsInvalidatingForExistingResource = function (state, _a, value) {
    var type = _a.type, id = _a.id;
    if (value === void 0) { value = null; }
    var idx = findIndex(state[type].data, function (e) { return e.id === id && e.type === type; });
    var updatePath = [type, "data", idx, "isInvalidating"];
    return value === null
        ? imm(state).del(updatePath)
        : imm(state).set(updatePath, value);
};
export var ensureResourceTypeInState = function (state, type) {
    var path = [type, "data"];
    return hasOwnProperties(state, [type])
        ? state
        : imm(state)
            .set(path, [])
            .value();
};
//# sourceMappingURL=state-mutation.js.map